@@ -18,9 +18,6 @@
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 675 Mass Ave, Cambridge, MA 02139, USA.
- * 
- * $Id: dictP.h,v 1.18 2004/10/06 14:59:10 cheusov Exp $
- * 
  */
 
 #ifndef _DICTP_H_
@@ -43,7 +40,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#ifdef HAVE_WCTYPE_H
+#if defined(HAVE_WCTYPE_H) && defined(SYSTEM_UTF8_FUNCS)
 #include <wctype.h>
 #endif
 
@@ -51,26 +48,22 @@
 #include <ctype.h>
 #endif
 
-#ifndef __GNUC__
-#define __FUNCTION__ __FILE__
-#endif
-
 /* AIX requires this to be the first thing in the file.  */
-#if defined(__svr4__) && defined(__sgi__) && !HAVE_ALLOCA_H /* IRIX */
+#if defined(__IRIX__) && defined(__sgi__) && !HAVE_ALLOCA_H
 # undef HAVE_ALLOCA_H
 # define HAVE_ALLOCA_H 1
 #endif
 
 #if HAVE_ALLOCA_H
 # include <alloca.h>
-#else
-# ifdef _AIX
-# pragma alloca
-# else
-#  ifndef alloca /* predefined by HP cc +Olibcalls */
-#  if !defined(__svr4__) && !defined(__sgi__)	/* not on IRIX */
-    char *alloca ();
-#  endif
+#endif
+
+#ifndef HAVE_ALLOCA
+# ifndef alloca /* predefined by HP cc +Olibcalls */
+#  ifdef _AIX
+#    pragma alloca
+#  else
+     void *alloca(size_t size);
 #  endif
 # endif
 #endif
@@ -114,6 +107,10 @@ extern unsigned long int strtoul( const char *, char **, int );
 extern int snprintf(char *str, size_t size, const char *format, ...);
 #endif
 
+#if !HAVE_DAEMON
+extern int daemon(int nochdir, int noclose);
+#endif
+
 #if !HAVE_VSNPRINTF
 #include <stdarg.h>
 extern int vsnprintf(char *str, size_t size, const char *format, va_list ap);
@@ -131,17 +128,20 @@ typedef unsigned int wint_t;
 
 #if !HAVE_ISWALNUM
 extern int iswalnum__ (wint_t wc);
-#define iswalnum iswalnum__
+#else
+#define iswalnum__ iswalnum
 #endif
 
 #if !HAVE_ISWSPACE
 extern int iswspace__ (wint_t wc);
-#define iswspace iswspace__
+#else
+#define iswspace__ iswspace
 #endif
 
 #if !HAVE_TOWLOWER
 extern wint_t towlower__ (wint_t wc);
-#define towlower towlower__
+#else
+#define towlower__ towlower
 #endif
 
 #if HAVE_WCHAR_T
@@ -150,13 +150,19 @@ extern wint_t towlower__ (wint_t wc);
 typedef unsigned int wchar_t;
 #endif
 
-#if HAVE_LANGINFO_H
+#if HAVE_DECL_CODESET
 #include <langinfo.h>
 #else
 extern const char * nl_langinfo (int ITEM);
 #define CODESET 1234
 #endif
 
+#ifndef SYSTEM_UTF8_FUNCS
+#define MB_CUR_MAX__ 6
+#else
+#define MB_CUR_MAX__ MB_CUR_MAX
+#endif
+
 #if HAVE_MBSTATE_T
 #include <wchar.h>
 #else
@@ -172,23 +178,33 @@ size_t strlcat(char *dst, const char *src, size_t siz);
 #endif
 
 #if !HAVE_WCRTOMB
-extern size_t wcrtomb (char *s, wchar_t wc, mbstate_t *ps);
+extern size_t wcrtomb__ (char *s, wchar_t wc, mbstate_t *ps);
+#else
+#define wcrtomb__ wcrtomb
 #endif
 
 #if !HAVE_WCTOMB
-extern int wctomb (char *s, wchar_t wc);
+extern int wctomb__ (char *s, wchar_t wc);
+#else
+#define wctomb__ wctomb
 #endif
 
 #if !HAVE_MBRLEN
-extern size_t mbrlen (const char *s, size_t n, mbstate_t *ps);
+extern size_t mbrlen__ (const char *s, size_t n, mbstate_t *ps);
+#else
+#define mbrlen__ mbrlen
 #endif
 
 #if !HAVE_MBRTOWC
-extern size_t mbrtowc (wchar_t *pwc, const char *s, size_t n, mbstate_t *ps);
+extern size_t mbrtowc__ (wchar_t *pwc, const char *s, size_t n, mbstate_t *ps);
+#else
+#define mbrtowc__ mbrtowc
 #endif
 
 #if !HAVE_MBSTOWCS
-extern size_t mbstowcs (wchar_t *dest, const char *src, size_t n);
+extern size_t mbstowcs__ (wchar_t *dest, const char *src, size_t n);
+#else
+#define mbstowcs__ mbstowcs
 #endif
 
 #if !HAVE_SETENV
@@ -196,11 +212,13 @@ extern int setenv(const char *name, const char *value, int overwrite);
 #endif
 
 #if !HAVE_MBTOWC
-extern int mbtowc (wchar_t *pwc, const char *s, size_t n);
+extern int mbtowc__ (wchar_t *pwc, const char *s, size_t n);
+#else
+#define mbtowc__ mbtowc
 #endif
 
 #if !HAVE_WCWIDTH
-#define wcwidth(x) (1)
+#define wcwidth__(x) (1)
 #endif
 
 #if !HAVE_INITGROUPS
@@ -288,11 +306,6 @@ extern void     srandom( unsigned int );
 #include <sys/mman.h>
 #endif
 
-/* System dependent declarations: Many brain damaged systems don't provide
-declarations for standard library calls.  We provide them here for
-situations that we know about. */
-#include "decl.h"
-
 #if HAVE_LIMITS_H
 #include <limits.h>
 #endif
