@@ -17,7 +17,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  * 
- * $Id: maa.h,v 1.41 2003/12/30 16:28:20 cheusov Exp $
+ * $Id: maa.h,v 1.43 2006/05/27 14:25:58 cheusov Exp $
  */
 
 #ifndef _MAA_H_
@@ -437,8 +437,7 @@ extern void           dbg_unset_flag( dbg_Type flag );
 extern __inline__ int dbg_test( dbg_Type flag );
 extern void           dbg_list( FILE *stream );
 
-#define PRINTF(flag,arg)  if (dbg_test(flag)) { printf arg; }
-#define FPRINTF(flag,arg) if (dbg_test(flag)) { fprintf arg; }
+#define PRINTF(flag,arg)  if (dbg_test(flag)) { log_info arg; }
 
 /* flags.c */
 
@@ -535,6 +534,7 @@ extern void txt_soundex2(
 
 extern const char    *b64_encode( unsigned long val );
 extern unsigned long b64_decode( const char *val );
+extern unsigned long b64_decode_buf (const char *val, size_t len);
 
 /* base26.c */
 
